% Michael Koehle 2015
% This code is provided as is with no guaranty of any sort

clear *

%****** User Set Variables***************
folder = 'C:\Users\mkoehle\Documents\ART\Instructables\fake color 3d print\colorSlicerCode\'; % set to your working folder: 'c:\myfolder\'
outFolder = folder;
stlFileName= 'womanHead.stl'; 
imageName = '2p1 womanHeadAnterior.tif'; % your projection image. should have resolution = res (see below)
svgRoot = 'svg'; % the slice root name
layerThickness = .0719; % thickness between slices. Should be equal to one page + spacer height
pageSize = [11 8.5]; % [width height] in inches of paper to be printed on
imageDepth =.36; % how far back on the page to smear the image

svgSizeWidth = 24; % width of your svg document. This is equal to my laser cutter printer bed. 
svgSizeHeight = 18; % height
res = 72; % pts per inch and pixels per inch.  

%***HOLE POSITIIONS****
% add the coordinates and diameter of any guide holes. 
% if you need more than two, add hole{3}= [x,y,z], etc
% i would suggest running the code, opening some of the SVG files, and
% finding locations for the holes that will be present in all slices. 
diameter = 1/8; %in inches
holes{1} = [4.5,4.1];
holes{2} = [4.8,4.1];
holes{3} = [2.8,4.1];

%****** End User Set Variables***********
             
% Image Setup
rawImage = imread([folder imageName]);
imSize = size(rawImage);

% Read in Stl
[faces,vertices] = stlread([folder stlFileName]);
% the 3 values in a row of faces are 3 row #s in vertices that make up a triangle 

%get limits of vertices
xLimits = [min(vertices(:,1)) max(vertices(:,1))];
yLimits = [min(vertices(:,2)) max(vertices(:,2))];
zLimits = [min(vertices(:,3)) max(vertices(:,3))];

% calculate offsets so model will be centered in page, in xy plane
svgXoffset = (pageSize(1)-diff(xLimits))/2;
svgYoffset = (pageSize(2)-diff(yLimits))/2;

% translate model to positive space, so min = 0. Center in xy plane, and 
% flip accross y (this flip is because +y within xvg is down, when I want
% +y to point up
vertices(:,1) = vertices(:,1)+svgXoffset-xLimits(1); 
vertices(:,2) = -vertices(:,2)+svgYoffset+yLimits(2);
vertices(:,3) = vertices(:,3)-zLimits(1);


% get the z values from each triangle:
zPts = [vertices(faces(:,1),3) vertices(faces(:,2),3) vertices(faces(:,3),3)];
minZ = min(zPts,[], 2); % get the min z value for each triangle
maxZ = max(zPts,[], 2); % get the max z value for each triangle

%sort minimums and maximums 
[zValue, index] = sort(minZ); % [zMinimum triangle#]
lowPtsList = [zValue, index];
[zValue, index]  = sort(maxZ);
highPtsList = [zValue, index];

%define size of out image: outimage is based on the xy plane of the mesh. 
% x is width of page and so is columns
outSize = [round(diff(yLimits)*res) round(diff(xLimits)*res) 3];

%current plan Z
currentZ = lowPtsList (1,1);


sliceNum = 0;
spanList = [];
lowIndex = 0; 
highIndex = 0;

while currentZ <= highPtsList(end, 1) % move through all slices
    sliceNum =  sliceNum+1;
    outImage = ones(outSize)*255; % set outImage to white
    
%     % add border to clarify orientation
%     outImage(:,1) = 0;
%     outImage(1,:) = 0; 
    
    imageName = [outFolder, sprintf('%03d', sliceNum),'_Image', '.jpg']; %output image name for slice
    svgName = [outFolder, sprintf('%03d', sliceNum),'_',svgRoot, '_slice', '.svg']; %output svg name for slice
    initSvg(svgSizeWidth*res,svgSizeHeight*res, svgName); % initialize svg file 
    
    % if a triangle's low point is less than the current z height, add it to
    % a list of triangles 'spanList'. These are triangles that might
    % possibly span the slice's z height
    if lowIndex < length(lowPtsList) %was numel instead of lenght, not sure why
       while lowPtsList(lowIndex + 1, 1) <= currentZ
           spanList(end + 1) = lowPtsList(lowIndex + 1, 2);
           lowIndex = lowIndex + 1;
       end
    end

    % if a triangle's high pt is less than the current z, add it to  
    % a list of triangle's subtractList - these are trinagles that do not
    % span the z height. 
    if highIndex < length(highPtsList)
        subtractList = [];
        while highPtsList(highIndex + 1, 1) < currentZ
            subtractList(end + 1) = highPtsList(highIndex + 1, 2);
            highIndex = highIndex + 1;
        end
         % subtract the non spanning triangles from the list of spanning
         % triangles
        spanList = setdiff(spanList, subtractList);
    end
    
    %compute intersections of all current triangles with current Z
    currIntersect = {};
    for t = 1:length(spanList)
        trianglePts = vertices(faces(spanList(t),:), :); % get the 3 pts that make up the triangle
        ptsOnPlane = find(trianglePts(:,3)==currentZ);
        switch length(ptsOnPlane)
            case 3 % the triangle is on the slice plane. in this case we will not write any pts.
            case 2 % two points are on the the slice plane. the Intersects will be these two points
                x1 = trianglePts(ptsOnPlane(1),1);
                y1 = trianglePts(ptsOnPlane(1),2);
                x2 = trianglePts(ptsOnPlane(2),1);
                y2 = trianglePts(ptsOnPlane(2),2);
                makeSvgLine(x1,y1, x2, y2, svgName);
            case 1 % one point is on the slice plane. no points will be used from this triangle
            case 0 % no pts are coincient with slice plane. continue
                ptsBelow = find(trianglePts(:,3)<currentZ);
                ptsAbove = find(trianglePts(:,3)>currentZ);
                if length(ptsBelow) > 1 
                    line1 = [trianglePts(ptsBelow(1),:); trianglePts(ptsAbove(1),:)];
                    line2 = [trianglePts(ptsBelow(2),:); trianglePts(ptsAbove(1),:)];
                else
                    line1 = [trianglePts(ptsAbove(1),:); trianglePts(ptsBelow(1),:)];
                    line2 = [trianglePts(ptsAbove(2),:); trianglePts(ptsBelow(1),:)];
                end
                x1 = (currentZ - line1(1,3)) * (line1(2,1) - line1(1,1)) / (line1(2,3) - line1(1,3)) + line1(1,1);
                y1 = (currentZ - line1(1,3)) * (line1(2,2) - line1(1,2)) / (line1(2,3) - line1(1,3)) + line1(1,2);
                x2 = (currentZ - line2(1,3)) * (line2(2,1) - line2(1,1)) / (line2(2,3) - line2(1,3)) + line2(1,1);
                y2 = (currentZ - line2(1,3)) * (line2(2,2) - line2(1,2)) / (line2(2,3) - line2(1,3)) + line2(1,2);
                makeSvgLine(x1,y1, x2, y2, svgName);
        end
    end
    
    pause(0.2)
    
    for h=1:length(holes) % draw holes
        makeSvgCircle(holes{h}(1),holes{h}(2),diameter/2,svgName);
    end
    closeSvg(svgName);
        
    %****Slice the image******
    % fill output image with single row of pixels from sourceImage
    if  currentZ <= imSize(1)/res
        % get the row at the currentZ height:
        pixelRow = imSize(1)-floor(currentZ*res);
        mDepth = ceil(imageDepth*outSize(2));
        for n=1:imSize(2)
            % scan through the columns of the source image at the currentZ,
            % and set an entire row of the outImage to this single pixel.
            outImage(n,1:mDepth,1) = rawImage(pixelRow,n,1);
            outImage(n,1:mDepth,2) = rawImage(pixelRow,n,2);
            outImage(n,1:mDepth,3) = rawImage(pixelRow,n,3);
        end
        outImage = uint8(outImage);
        imwrite(outImage, imageName, 'jpg');
    end
    
    %move to next plan
     currentZ = currentZ + layerThickness;
end




